import { EventEmitter, Component, ViewEncapsulation, ElementRef, Input, Output, NgModule } from '@angular/core';
import { Grid } from 'gridjs';
import { CommonModule } from '@angular/common';

const GRIDJS_EVENTS = [
    'beforeLoad',
    'cellClick',
    'load',
    'rowClick',
];
const GRIDJS_PROPS = [
    'eventEmitter',
    'dispatcher',
    'plugin',
    'data',
    'server',
    'header',
    'from',
    'storage',
    'pipeline',
    'autoWidth',
    'width',
    'height',
    'translator',
    'style',
    'className',
    'fixedHeader',
    'columns',
    'search',
    'pagination',
    'sort',
    'language',
];

class GridJsAngularComponent {
    constructor(elementDef) {
        this.listeners = new Map();
        this.plugins = [];
        // events
        this.beforeLoad = new EventEmitter(true);
        // renamed load event to avoid conflict with native load event
        this.gridLoad = new EventEmitter(true);
        this.cellClick = new EventEmitter(true);
        this.rowClick = new EventEmitter(true);
        this.nativeElement = elementDef.nativeElement;
    }
    ngAfterViewInit() {
        this.gridInstance = new Grid(this.getConfig(this.gridConfig));
        this.registerPlugins();
        this.registerEvents();
        this.gridInstance.render(this.nativeElement);
        this.initialized = true;
    }
    ngOnChanges(changes) {
        if (this.initialized) {
            this.updateConfig(this.gridConfig);
        }
    }
    ngOnDestroy() {
        if (this.initialized) {
            if (this.gridInstance) {
                this.unregisterEvents();
                this.gridInstance = null;
            }
        }
    }
    // public api to interact with grid instance
    getGridInstance() {
        return this.gridInstance;
    }
    updateConfig(config) {
        this.gridInstance.updateConfig(this.getConfig(config)).forceRender();
    }
    registerEvents() {
        for (const event of GRIDJS_EVENTS) {
            const emitter = event === 'load'
                ? this.gridLoad
                : this[event];
            const listener = (...args) => emitter.emit(args);
            this.listeners.set(event, listener);
            if (emitter) {
                this.gridInstance.on(event, listener);
            }
        }
    }
    unregisterEvents() {
        for (const [event, listener] of this.listeners.entries()) {
            this.gridInstance.off(event, listener);
        }
    }
    registerPlugins() {
        for (const plugin of this.plugins) {
            if (!this.gridInstance.plugin.get(plugin.id)) {
                this.gridInstance.plugin.add(plugin);
            }
        }
    }
    getConfig(config) {
        const newConfig = Object.assign({}, config);
        for (const [key, value] of Object.entries(this)) {
            if (GRIDJS_PROPS.includes(key)) {
                newConfig[key] = value;
            }
        }
        this.gridConfig = newConfig;
        return newConfig;
    }
}
GridJsAngularComponent.decorators = [
    { type: Component, args: [{
                selector: 'gridjs-angular',
                template: '',
                encapsulation: ViewEncapsulation.None
            },] }
];
GridJsAngularComponent.ctorParameters = () => [
    { type: ElementRef }
];
GridJsAngularComponent.propDecorators = {
    gridConfig: [{ type: Input }],
    plugins: [{ type: Input }],
    eventEmitter: [{ type: Input }],
    dispatcher: [{ type: Input }],
    plugin: [{ type: Input }],
    data: [{ type: Input }],
    server: [{ type: Input }],
    header: [{ type: Input }],
    from: [{ type: Input }],
    storage: [{ type: Input }],
    pipeline: [{ type: Input }],
    autoWidth: [{ type: Input }],
    width: [{ type: Input }],
    height: [{ type: Input }],
    translator: [{ type: Input }],
    style: [{ type: Input }],
    className: [{ type: Input }],
    fixedHeader: [{ type: Input }],
    columns: [{ type: Input }],
    search: [{ type: Input }],
    pagination: [{ type: Input }],
    sort: [{ type: Input }],
    language: [{ type: Input }],
    beforeLoad: [{ type: Output }],
    gridLoad: [{ type: Output }],
    cellClick: [{ type: Output }],
    rowClick: [{ type: Output }]
};

class GridJsAngularModule {
}
GridJsAngularModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [GridJsAngularComponent],
                exports: [GridJsAngularComponent],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { GRIDJS_EVENTS, GRIDJS_PROPS, GridJsAngularComponent, GridJsAngularModule };
//# sourceMappingURL=gridjs-angular.js.map
